use std::ffi::CString;

use anyhow::Result;
use ilhook::x64::Registers;

use crate::marshal;

use super::{MhyContext, MhyModule, ModuleType};

pub struct Sdk;

const SDKUTIL_RSAENCRYPT: usize = 0x1334B980;
const CUSTOM_KEY: &str = include_str!("../../sdk_public_key.xml");

impl MhyModule for MhyContext<Sdk> {
    unsafe fn init(&mut self) -> Result<()> {
        self.interceptor.attach(
            self.assembly_base + SDKUTIL_RSAENCRYPT,
            on_sdkutil_rsaencrypt,
        )
    }

    unsafe fn de_init(&mut self) -> Result<()> {
        Ok(())
    }

    fn get_module_type(&self) -> super::ModuleType {
        ModuleType::Sdk
    }
}

unsafe extern "win64" fn on_sdkutil_rsaencrypt(reg: *mut Registers, _: usize) {
    println!("[SDK] SDKUtil::RSAEncrypt(), replacing the key");
    (*reg).rcx = marshal::ptr_to_string_ansi(CString::new(CUSTOM_KEY).unwrap().as_c_str()) as u64;
}
