use crate::util;

use super::{MhyContext, MhyModule, ModuleType};
use anyhow::Result;
use ilhook::x64::Registers;
use lazy_static::lazy_static;

const CONVERT_FROM_BASE64_CHAR_PTR: usize = 0x12A1CF90;
const ORIGIN_MODULUS: &str = "7kySpSkK2pCgKSw5A6nY/xtr2XI9+nFpdcdPy49eTpekeftrddtGLXoFt4xKkRR4ovN9qtenPLQbEF5gDbS5BQ5sk4yP0rJSFBK+ol4yx7JX2Nm9MxE+Zz8zb0iM8Hm6AewHIoLadpB06H6Hdh4XZKYmUPueH4yz1ZLFQDGzdLU=";
const CUSTOM_MODULUS: &str =
    "d8zyiJhwWK1Gocou1iynAKbH9f27uAtqFWec+Rbr6G9JFzyAsqH2G7iKqPV60lItLgJ/jvcVyzPeE9eQ1h6Fjw==";

pub struct Security;

impl MhyModule for MhyContext<Security> {
    unsafe fn init(&mut self) -> Result<()> {
        self.interceptor.attach(
            self.assembly_base + CONVERT_FROM_BASE64_CHAR_PTR,
            on_from_base64_char_ptr,
        )
    }

    unsafe fn de_init(&mut self) -> Result<()> {
        Ok(())
    }

    fn get_module_type(&self) -> ModuleType {
        ModuleType::Security
    }
}

unsafe extern "win64" fn on_from_base64_char_ptr(reg: *mut Registers, _: usize) {
    lazy_static! {
        static ref MODULUS_VEC: Vec<u16> = util::wide_str(CUSTOM_MODULUS);
    };

    let char_ptr = (*reg).rcx as *const u8;
    let strlen = (*reg).rdx as usize;

    if strlen == 172 {
        let slice = std::slice::from_raw_parts(char_ptr.cast::<u16>(), strlen);
        let modulus = String::from_utf16(slice).unwrap();

        if modulus == ORIGIN_MODULUS {
            println!("[SECURITY] replaced server public key modulus");

            (*reg).rcx = MODULUS_VEC.as_slice().as_ptr() as u64;
            (*reg).rdx = 88;
        }
    }
}
